package fr.emse.gitlab.saref.jobs;

import java.io.File;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import fr.emse.gitlab.saref.entities.tests.TestCase;
import fr.emse.gitlab.saref.entities.tests.TestSuite;
import fr.emse.gitlab.saref.entities.tests.TestSuites;
import fr.emse.gitlab.saref.entities.tests.TestCase.Status;
import fr.emse.gitlab.saref.jobs.JobRunner;

public abstract class AbstractJobRunner<T> implements JobRunner<T> {
	
	private final TestSuite testSuite;
	protected final File directory;
	
	public AbstractJobRunner(String testName, File dir) {
		this.directory = dir;
		if(testName == null) {
			testName = this.getClass().getCanonicalName();
		}
		testSuite = new TestSuite(testName);
	}	
	
	public final T doJob(TestSuites testSuites) {
		T result = getValue();
		testSuites.addTestsuite(testSuite);
		return result;
	}

	protected T getValue() {
		doJob0();
		return null;
	};

	protected void doJob0() {
	};

	protected void error(String name, Exception ex) {
		TestCase tc = new TestCase(name);
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		ex.printStackTrace(pw);
		tc.setSystemErr(sw.toString());
		tc.setError(ex.getClass().getCanonicalName(), ex.getMessage());
		testSuite.addTestcase(tc);
	}

	protected void failure(String name, Exception ex) {
		TestCase tc = new TestCase(name);
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		ex.printStackTrace(pw);
		tc.setSystemErr(sw.toString());
		tc.setFailure(ex.getClass().getCanonicalName(), ex.getMessage());
		testSuite.addTestcase(tc);
	}

	protected void error(String name, String type, String message) {
		TestCase tc = new TestCase(name);
		tc.setError(type, message);
		testSuite.addTestcase(tc);
	}
	
	protected void error(String name) {
		TestCase tc = new TestCase(name);
		tc.setStatus(Status.ERROR);
		testSuite.addTestcase(tc);
	}

	protected void warning(String name) {
		TestCase tc = new TestCase(name);
		tc.setStatus(Status.WARNING);
		testSuite.addTestcase(tc);
	}

	protected void failure(String name) {
		TestCase tc = new TestCase(name);
		tc.setStatus(Status.FAILURE);
		testSuite.addTestcase(tc);
	}

	protected void failure(String name, String type, String message) {
		TestCase tc = new TestCase(name);
		tc.setFailure(type, message);
		testSuite.addTestcase(tc);
	}

	protected void success(String name) {
		testSuite.addTestcase(new TestCase(name));
	}
	

	final protected String getRepoName() {
		String REGEX = "^(?<ext>saref-core|saref4[a-z][a-z][a-z][a-z])";
		String name = directory.getName();
		Pattern pattern = Pattern.compile(REGEX);
		Matcher matcher = pattern.matcher(name);
		if (!matcher.find()) {
			error(String.format(
					"The SAREF pipeline must be run inside a directory whose name begins with `saref-core`, or `saref4abcd` (where abcd can be any sequence of four letters). Got: %s",
					name));
			return null;
		}
		return matcher.group("ext");
	}


}
