package fr.emse.gitlab.saref.jobs;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.util.HashMap;
import java.util.Map;

import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.riot.Lang;

public class CheckExamples extends AbstractJobRunner<Void> {

	static final Map<String, String> PREFIXES = new HashMap<String, String>();
	static {
		PREFIXES.put("owl", "http://www.w3.org/2002/07/owl#");
		PREFIXES.put("rdfs", "http://www.w3.org/2000/01/rdf-schema#");
		PREFIXES.put("xsd", "http://www.w3.org/2001/XMLSchema#");
		PREFIXES.put("dcterms", "http://purl.org/dc/terms/");
		PREFIXES.put("vann", "http://purl.org/vocab/vann/");
		PREFIXES.put("foaf", "http://xmlns.com/foaf/0.1/");
		PREFIXES.put("schema", "http://schema.org/");
		PREFIXES.put("rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
		PREFIXES.put("voaf", "http://purl.org/vocommons/voaf#");
		PREFIXES.put("dce", "http://purl.org/dc/elements/1.1/");
		PREFIXES.put("dct", "http://purl.org/dc/terms/");
		PREFIXES.put("xml", "http://www.w3.org/XML/1998/namespace/");
		PREFIXES.put("saref", "https://saref.etsi.org/core/");
	}
	
	public CheckExamples(File dir) {
		super("Check examples", dir);
	}

	@Override
	protected void doJob0() {
//		String repoName = getRepoName();
		checkExamples();
	}

	private void checkExamples() {
		File dir = new File(directory, "examples");
		try {
			Files.walk(dir.toPath()).filter(p -> {
				return p.endsWith(".ttl");
			}).forEach(p -> {
				File exampleFile = p.toFile();
				Model model = ModelFactory.createDefaultModel();
				try (FileInputStream input = new FileInputStream(exampleFile)) {
					model.read(input, null, Lang.TTL.getLabel());
				} catch (Exception ex) {
					failure(String.format("Exception while reading the example file %s", exampleFile.toString()),
							ex.getClass().getSimpleName(), ex.getMessage());
				}
			});
		} catch (IOException ex) {
			error("Exception while walking the example directory", ex);
		}
	}

}
