package fr.emse.gitlab.saref.jobs;

import java.io.File;
import java.io.FileOutputStream;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.apache.jena.query.Dataset;
import org.apache.jena.query.ReadWrite;
import org.apache.jena.rdf.model.Model;
import org.semanticweb.owlapi.model.IRI;
import org.semanticweb.owlapi.model.OWLOntologyIRIMapper;
import org.semanticweb.owlapi.util.SimpleIRIMapper;

import fr.emse.gitlab.saref.Constants;
import fr.emse.gitlab.saref.utils.Languages;
import fr.emse.gitlab.saref.vocabs.EX;


public class WriteRDFFiles extends AbstractJobRunner<Set<OWLOntologyIRIMapper>> {

	private final Dataset dataset;
	
	public WriteRDFFiles(File directory, Dataset dataset) {
		super("Write RDF files", directory);
		this.dataset = dataset;
	}
	
	@Override
	public Set<OWLOntologyIRIMapper> getValue() {
		File siteDir = new File(directory, Constants.SITE_DIR);
		Set<OWLOntologyIRIMapper> mappers = new HashSet<>();
		
		dataset.begin(ReadWrite.READ);
		for (Iterator<String> it = dataset.listNames(); it.hasNext();) {
			String name = it.next();
			if (name.equals(EX.config)) {
				continue;
			}
			Model model = dataset.getNamedModel(name);
			String fileName = name.substring(Constants.BASE.length());
			if (fileName.endsWith("/")) {
				// is ontology
				fileName = fileName + fileName.substring(0, fileName.indexOf("/"));
				// register IRIMapper
				File ttlFile = new File(siteDir, fileName + ".ttl");
				OWLOntologyIRIMapper mapper = new SimpleIRIMapper(
						IRI.create(name),
						IRI.create(ttlFile));
				mappers.add(mapper);
			}
			try {
				FileUtils.forceMkdir(new File(siteDir, fileName).getParentFile());
				for (Languages l : Languages.values()) {
					File file = new File(siteDir, String.format("%s.%s", fileName, l.getExt()));
					try (FileOutputStream fos = new FileOutputStream(file)) {
						model.write(fos, l.getLang());
					}
				}
			} catch(Exception ex) {
				failure(String.format("Failed to write the model for %s", fileName), ex);
			}
		}
		dataset.end();
		
		return mappers;
	}
}
